package ch.zhaw.gartenverwaltung.bootstrap;

import ch.zhaw.gartenverwaltung.HelloApplication;
import ch.zhaw.gartenverwaltung.MainFXMLController;
import ch.zhaw.gartenverwaltung.io.CropList;
import ch.zhaw.gartenverwaltung.io.JsonCropList;
import ch.zhaw.gartenverwaltung.io.JsonPlantList;
import ch.zhaw.gartenverwaltung.io.JsonTaskList;
import ch.zhaw.gartenverwaltung.io.PlantList;
import ch.zhaw.gartenverwaltung.io.TaskList;
import ch.zhaw.gartenverwaltung.models.Garden;
import ch.zhaw.gartenverwaltung.models.GardenSchedule;
import ch.zhaw.gartenverwaltung.models.PlantListModel;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class AppLoader {
    /**
     * Caching the panes
     */
    private final Map<String, Pane> panes = new HashMap<>();

    /**
     * Application-wide dependencies
     */
    private final PlantList plantList = new JsonPlantList();
    private final CropList cropList = new JsonCropList();
    private final TaskList taskList = new JsonTaskList();

    private final GardenSchedule gardenSchedule = new GardenSchedule(taskList, plantList);
    private final Garden garden = new Garden(gardenSchedule, cropList);
    private final MainFXMLController mainController;


    public AppLoader(MainFXMLController mainController) throws IOException {
        this.mainController = mainController;
    }


    public Pane loadPane(String fxmlFile) throws IOException {
        Pane pane = panes.get(fxmlFile);
        if (pane == null) {
            loadAndCacheFxml(fxmlFile);
            pane = panes.get(fxmlFile);
        }
        return pane;
    }

    public Object loadSceneToStage(String fxmlFile, Stage appendee) throws IOException {
        FXMLLoader loader = new FXMLLoader(Objects.requireNonNull(HelloApplication.class.getResource(fxmlFile)));
        Pane root = loader.load();
        appendee.setScene(new Scene(root));
        Object controller = loader.getController();
        annotationInject(controller);
        return controller;
    }

    public void loadAndCacheFxml(String fxmlFile) throws IOException {
        FXMLLoader loader = new FXMLLoader(Objects.requireNonNull(HelloApplication.class.getResource(fxmlFile)));
        AnchorPane anchorPane = loader.load();
        panes.put(fxmlFile, anchorPane);
        annotationInject(loader.getController());
    }

    public void annotationInject(Object controller) {
        Arrays.stream(controller.getClass().getDeclaredFields())
                .filter(field -> field.isAnnotationPresent(Inject.class))
                .forEach(field -> {
                    field.setAccessible(true);
                    try {
                        field.set(controller, getAppDependency(field.getType()));
                    } catch (IllegalAccessException e) {
                        throw new RuntimeException(e);
                    }
                    field.setAccessible(false);
                });

        Arrays.stream(controller.getClass().getMethods())
                .filter(method -> method.isAnnotationPresent(AfterInject.class))
                .forEach(afterInjectMethod -> {
                    if (afterInjectMethod.getParameterCount() == 0) {
                        try {
                            afterInjectMethod.invoke(controller);
                        } catch (IllegalAccessException | InvocationTargetException e) {
                            e.printStackTrace();
                        }
                    }
                });
    }

    private Object getAppDependency(Class<?> type) {
        return switch (type.getSimpleName()) {
            case "Garden" -> garden;
            case "PlantList" -> plantList;
            case "PlantListModel" -> new PlantListModel(plantList);
            case "GardenSchedule" -> gardenSchedule;
            case "MainFXMLController" -> mainController;
            case "AppLoader" -> this;
            default -> null;
        };
    }
}
